@@ -66,9 +66,9 @@ class AAPIObject:
                       file_path: str = None, delete_afterwards: bool = True, open_in_browser: str = None) -> RunMonitor:        
         # Import circular dependency
         from ctm_python_client.core.workflow import Workflow, WorkflowDefaults
-        from aapi import Job, Folder
+        from aapi import Folder
         
-        if isinstance(self, Job) or (hasattr(self, 'job_list') and self.job_list is not None and len(self.job_list) > 0):
+        if (hasattr(self, '_type') and 'Job' in self._type) or (hasattr(self, 'job_list') and self.job_list is not None and len(self.job_list) > 0):
             try:
                 on_demand_workflow = Workflow(
                     environment,
@@ -85,16 +85,19 @@ class AAPIObject:
                 else:
                     on_demand_workflow.add(self, inpath=inpath)
                 
-                on_demand_workflow.run_on_demand(
+                return on_demand_workflow.run_on_demand(
                     skip_login=skip_login, 
                     file_path=file_path, 
                     delete_afterwards=delete_afterwards,
                     open_in_browser=open_in_browser
                     )
             except Exception as e:
-                errors = [err.get('message', '') + ' ' + err.get('item', '')
-                    for err in json.loads(e.body)['errors']]
-                raise RuntimeError(f"AAPI request failed: {', '.join(errors)}")
+                if e.body:
+                    errors = [err.get('message', '') + ' ' + err.get('item', '')
+                        for err in json.loads(e.body)['errors']]
+                    raise RuntimeError(f"AAPI request failed: {', '.join(errors)}")
+                else:
+                    raise e
             finally:
                 on_demand_workflow.clear_all()
         else:
